require 'io/mixins'
class IOLoop

  include IO::Editable

  attr_reader :string

  def initialize(string = "")
    @string = string
    @pos = 0
  end

  def sysseek(offset, whence = IO::SEEK_SET)
    @pos = case whence
           when IO::SEEK_SET then offset
           when IO::SEEK_CUR then @pos + offset
           when IO::SEEK_END then @string.length + offset
           end % @string.size
  end

  # def eof
    # @string.nil? || @string.empty?
  # end

  def putchar(char)
    @string[@pos,1] = char.respond_to?(:chr) ? char.chr : char[0,0]
    @pos += 1
    @pos %= @string.length
    char
  end
  private :putchar

  def syswrite(string)
    string.each_byte do |byte|
      putchar(byte)
    end
  end

  def sysread(length)
    raise IOError if @string.length < length
    if length <= @string.length - @pos
      out = @string[@pos,length]
      @pos += out.length
      @pos %= @string.length
      return out
    else
      out = @string[@pos..-1]
      @pos = 0
      out << @string[0,length-out.length]
    end
  end

  private
  # def ensure_open
    # raise IOError, "closed stream" if closed?
  # end

  alias close_read close
  alias close_write close

end
